@@ -65,6 +65,7 @@ void Department::loadFromDepartmentNode(DepartmentNode* treeNode)
         QSharedPointer<SubdepartmentData> subdept(new SubdepartmentData);
         subdept->id = node->id();
         subdept->label = node->label();
+        subdept->allLabel = node->allLabel();
         subdept->hasChildren = node->hasSubdepartments();
         subdept->isActive = false;
         m_subdepartments.append(subdept);
@@ -122,12 +123,23 @@ QVariant Department::data(const QModelIndex& index, int role) const
     switch (role) {
         case RoleNavigationId: return data->id;
         case RoleLabel: return data->label;
+        case RoleAllLabel: return data->allLabel;
         case RoleHasChildren: return data->hasChildren;
         case RoleIsActive: return data->isActive;
         default: return QVariant();
     }
 }
 
+QSharedPointer<SubdepartmentData> Department::findSubdepartment(const QString &id) const
+{
+    for (auto sub: m_subdepartments) {
+        if (sub->id == id) {
+            return sub;
+        }
+    }
+    return QSharedPointer<SubdepartmentData>();
+}
+
 int Department::rowCount(const QModelIndex& parent) const
 {
     Q_UNUSED(parent);
