@@ -38,6 +38,7 @@
 #include <unity/scopes/ScopeMetadata.h>
 #include <unity/shell/scopes/ScopeInterface.h>
 
+#include "filters.h"
 #include "collectors.h"
 #include "departmentnode.h"
 #include "department.h"
@@ -129,6 +130,11 @@ public:
     unity::shell::scopes::ScopeInterface::Status status() const override;
     unity::shell::scopes::CategoriesInterface* categories() const override;
     unity::shell::scopes::SettingsModelInterface* settings() const override;
+    unity::shell::scopes::FiltersInterface* filters() const override;
+    unity::shell::scopes::FilterBaseInterface* primaryNavigationFilter() const override;
+    QString primaryNavigationTag() const override;
+    int activeFiltersCount() const override;
+
     bool require_child_scopes_refresh() const;
     void update_child_scopes();
     QString searchQuery() const override;
@@ -137,8 +143,6 @@ public:
     bool isActive() const override;
     QString currentNavigationId() const override;
     bool hasNavigation() const override;
-    QString currentAltNavigationId() const override;
-    bool hasAltNavigation() const override;
     QVariantMap customizations() const override;
 
     /* setters */
@@ -153,10 +157,11 @@ public:
     Q_INVOKABLE void cancelActivation() override;
     Q_INVOKABLE void closeScope(unity::shell::scopes::ScopeInterface* scope) override;
     Q_INVOKABLE unity::shell::scopes::NavigationInterface* getNavigation(QString const& id) override;
-    Q_INVOKABLE unity::shell::scopes::NavigationInterface* getAltNavigation(QString const& id) override;
-    Q_INVOKABLE void setNavigationState(QString const& navId, bool altNavigation) override;
+    Q_INVOKABLE void setNavigationState(QString const& navId) override;
     Q_INVOKABLE void performQuery(QString const& cannedQuery) override;
     Q_INVOKABLE void refresh() override;
+    Q_INVOKABLE void resetPrimaryNavigationTag() override;
+    Q_INVOKABLE void resetFilters() override;
 
     void setScopeData(unity::scopes::ScopeMetadata const& data);
     void handleActivation(std::shared_ptr<unity::scopes::ActivationResponse> const&, unity::scopes::Result::SPtr const&, QString const& categoryId="");
@@ -191,6 +196,7 @@ private Q_SLOTS:
     void flushUpdates(bool finalize = false);
     void metadataRefreshed();
     void departmentModelDestroyed(QObject* obj);
+    void filterStateChanged();
     void previewModelDestroyed(QObject *obj);
 
 protected:
@@ -206,12 +212,14 @@ protected:
 
 private:
     static void updateNavigationModels(DepartmentNode* rootNode, QMultiMap<QString, Department*>& navigationModels, QString const& activeNavigation);
-    static QString buildQuery(QString const& scopeId, QString const& searchQuery, QString const& departmentId, QString const& primaryFilterId, QString const& primaryOptionId);
+    static QString buildQuery(QString const& scopeId, QString const& searchQuery, QString const& departmentId, unity::scopes::FilterState const& filterState);
     void setScopesInstance(Scopes*);
     void startTtlTimer();
     void setCurrentNavigationId(QString const& id);
     void setFilterState(unity::scopes::FilterState const& filterState);
     void processSearchChunk(PushEvent* pushEvent);
+    void processPrimaryNavigationTag(QString const &targetDepartmentId);
+    void processActiveFiltersCount();
     void setCannedQuery(unity::scopes::CannedQuery const& query);
     void executeCannedQuery(unity::scopes::CannedQuery const& query, bool allowDelayedActivation);
     void handlePreviewUpdate(unity::scopes::Result::SPtr const& result, unity::scopes::PreviewWidgetList const& widgets);
@@ -227,16 +235,16 @@ private:
     QString m_noResultsHint;
     QString m_formFactor;
     QString m_currentNavigationId;
-    QString m_currentAltNavigationId;
+    QString m_primaryNavigationTag;
     QVariantMap m_customizations;
     std::unique_ptr<unity::scopes::Variant> m_queryUserData;
+    int m_activeFiltersCount;
     bool m_isActive;
     bool m_searchInProgress;
     bool m_activationInProgress;
     bool m_resultsDirty;
     bool m_delayedSearchProcessing;
     bool m_hasNavigation;
-    bool m_hasAltNavigation;
     bool m_favorite;
     bool m_initialQueryDone;
 
@@ -248,20 +256,18 @@ private:
     std::shared_ptr<unity::scopes::ActivationResponse> m_delayedActivation;
     unity::scopes::Department::SCPtr m_rootDepartment;
     unity::scopes::Department::SCPtr m_lastRootDepartment;
-    unity::scopes::OptionSelectorFilter::SCPtr m_sortOrderFilter;
-    unity::scopes::OptionSelectorFilter::SCPtr m_lastSortOrderFilter;
     unity::scopes::FilterState m_filterState;
-    unity::scopes::FilterState m_receivedFilterState;
     unity::shell::scopes::ScopeInterface::Status m_status;
+    QList<unity::scopes::FilterBase::SCPtr> m_receivedFilters;
+    QScopedPointer<Filters> m_filters;
+
     QScopedPointer<SettingsModel> m_settingsModel;
     QSharedPointer<DepartmentNode> m_departmentTree;
-    QSharedPointer<DepartmentNode> m_altNavTree;
     QTimer m_typingTimer;
     QTimer m_searchProcessingDelayTimer;
     QTimer m_invalidateTimer;
     QList<std::shared_ptr<unity::scopes::CategorisedResult>> m_cachedResults;
     QMultiMap<QString, Department*> m_departmentModels;
-    QMultiMap<QString, Department*> m_altNavModels;
     QMap<Department*, QString> m_inverseDepartments;
     QMetaObject::Connection m_metadataConnection;
     QSharedPointer<LocationService> m_locationService;
