@@ -34,17 +34,19 @@ class Q_DECL_EXPORT UbuntuLocationService : public LocationService
     Q_OBJECT
 
 public:
-    UbuntuLocationService(GeoIp::Ptr geoIp = GeoIp::Ptr(new GeoIp));
+    class TokenImpl;
+
+    UbuntuLocationService(const GeoIp::Ptr& geoIp = GeoIp::Ptr(new GeoIp));
 
     virtual ~UbuntuLocationService();
 
     unity::scopes::Location location() const override;
 
+    bool hasLocation() const override;
+
     bool isActive() const override;
 
-    void activate() override;
-
-    void deactivate() override;
+    QSharedPointer<Token> activate() override;
 
 Q_SIGNALS:
     void enqueueActivate();
