@@ -31,7 +31,6 @@ namespace scopes
 {
     class PreviewWidgetModelInterface;
     class PreviewModelInterface;
-    class PreviewStackInterface;
 }
 }
 namespace scopeharness
@@ -54,8 +53,6 @@ struct PreviewWidgetArguments
     std::shared_ptr<view::ResultsView> resultsView;
 
     std::shared_ptr<view::PreviewView> previewView;
-
-    std::shared_ptr<unity::shell::scopes::PreviewStackInterface> previewStack;
 };
 }
 }
