@@ -23,11 +23,12 @@
 #include <scope-harness/view/results-view.h>
 #include <scope-harness/test-utils.h>
 
+#include <Unity/previewmodel.h>
+
 #include <unity/shell/scopes/PreviewModelInterface.h>
 #include <unity/shell/scopes/PreviewWidgetModelInterface.h>
 
 #include <Unity/utils.h>
-#include <Unity/previewstack.h>
 
 #include <QDebug>
 #include <QSignalSpy>
@@ -57,8 +58,6 @@ struct PreviewWidget::_Priv
     weak_ptr<view::ResultsView> m_resultsView;
 
     weak_ptr<view::PreviewView> m_previewView;
-
-    std::shared_ptr<unity::shell::scopes::PreviewStackInterface> m_previewStack;
 };
 
 PreviewWidget::PreviewWidget(const internal::PreviewWidgetArguments& arguments) :
@@ -69,7 +68,6 @@ PreviewWidget::PreviewWidget(const internal::PreviewWidgetArguments& arguments)
     p->m_index = arguments.index;
     p->m_resultsView = arguments.resultsView;
     p->m_previewView = arguments.previewView;
-    p->m_previewStack = arguments.previewStack;
 }
 
 
@@ -91,7 +89,6 @@ PreviewWidget& PreviewWidget::operator=(const PreviewWidget& other)
     p->m_index = other.p->m_index;
     p->m_resultsView = other.p->m_resultsView;
     p->m_previewView = other.p->m_previewView;
-    p->m_previewStack = other.p->m_previewStack;
     return *this;
 }
 
@@ -127,9 +124,8 @@ sc::Variant PreviewWidget::data() const
 
 view::AbstractView::SPtr PreviewWidget::trigger(const string& name, const sc::Variant& v)
 {
-    auto ps = std::dynamic_pointer_cast<ng::PreviewStack>(p->m_previewStack);
-    TestUtils::throwIfNot(bool(ps), "No preview stack");
-    TestUtils::throwIfNot(bool(ps->associatedScope()), "Preview stack has no associated scope");
+    auto ps = dynamic_cast<ng::PreviewModel*>(p->m_previewModel);
+    TestUtils::throwIfNot(bool(ps->associatedScope()), "Preview model has no associated scope");
     QSignalSpy showDashSpy(ps->associatedScope(), SIGNAL(showDash()));
 
     QVariant widgetData;
@@ -153,7 +149,7 @@ view::AbstractView::SPtr PreviewWidget::trigger(const string& name, const sc::Va
         widgetData = ng::scopeVariantToQVariant(v);
     }
 
-    Q_EMIT p->m_previewModel->triggered(
+    Q_EMIT ps->triggered(
             QString::fromStdString(id()), QString::fromStdString(name),
             widgetData.toMap());
 
