@@ -146,6 +146,30 @@ public:
             res.set_title("result for: \"" + query_ + "\"");
             reply->push(res);
         }
+        else if (query_ == "result-action")
+        {
+            CategoryRenderer minimal_rndr(R"({"schema-version": 1, "components": {"title": "title"}})");
+            auto cat = reply->register_category("cat1", "Category 1", "", minimal_rndr);
+            CategorisedResult res(cat);
+
+            res.set_uri("test:result-action");
+            res.set_title("result for: \"" + query_ + "\"");
+
+            // Add result actions
+            VariantBuilder builder;
+            builder.add_tuple({
+                    {"id", Variant("action1")},
+                    {"icon", Variant("icon1")},
+                    {"label", Variant("Action1")},
+            });
+            builder.add_tuple({
+                    {"id", Variant("action2")},
+                    {"icon", Variant("icon2")},
+                    {"label", Variant("Action2")},
+            });
+            res["social_attributes"] = builder.end(); // TODO: verify with shell
+            reply->push(res);
+        }
         else if (query_ == "update-preview")
         {
             CategoryRenderer minimal_rndr(R"({"schema-version": 1, "components": {"title": "title"}})");
@@ -445,6 +469,13 @@ public:
     {
     }
 
+    MyActivation(Result const& result, ActionMetadata const& metadata, ActivationResponse::Status status, std::string const& action_id) :
+        ActivationQueryBase(result, metadata),
+        status_(status),
+        action_id_(action_id)
+    {
+    }
+
     ~MyActivation() noexcept
     {
     }
@@ -460,7 +491,12 @@ public:
 
     virtual ActivationResponse activate() override
     {
-        if (status_ == ActivationResponse::Status::PerformQuery) {
+        if (status_ == ActivationResponse::Status::UpdateResult) {
+            Result updatedRes(result());
+            updatedRes["actionId"] = Variant(action_id_);
+            return ActivationResponse(updatedRes);
+        }
+        else if (status_ == ActivationResponse::Status::PerformQuery) {
             auto resp = ActivationResponse(CannedQuery(result()["scope-id"].get_string()));
             return resp;
         } else {
@@ -473,6 +509,7 @@ public:
 private:
     Variant extra_data_;
     ActivationResponse::Status status_;
+    std::string action_id_;
 };
 
 
@@ -554,6 +591,11 @@ public:
         }
         return ActivationQueryBase::UPtr(new MyActivation(result, meta));
     }
+
+    ActivationQueryBase::UPtr activate_result_action(Result const& result, ActionMetadata const& meta, std::string const& action_id) override
+    {
+        return ActivationQueryBase::UPtr(new MyActivation(result, meta, ActivationResponse::UpdateResult, action_id));
+    }
 };
 
 extern "C"
